#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAXOP 150       // max size of operand or operator
#define NUMBER '0'      // signal that a number was found

int getop(char []);
void push(double);
void pushl(long int);
double pop(void);
void stackc(void);

/* reverse Polish calculator */
main()
{
      int type;
      double op2;
      long int tempop, tempop2;
      char s[MAXOP];
      
      printf("Reverse Polish calculator by UnKnown X.\n");
      printf("(Press \'y\' any time to read the description.)\n\n");
      
      while ((type = getop(s)) != EOF) {
            switch (type) {
            case NUMBER:
                 push(atof(s));
                 break;
            case '+':
                 push(pop() + pop());
                 break;
            case '*':
                 push(pop() * pop());
                 break;
            case '-':
                 op2 = pop();
                 push(pop() - op2);
                 break;
            case '/':
                 op2 = pop();
                 push(pop() / op2);
                 break;
            case '^':                       // power
                 push(pow(pop(),pop()));
                 break;
            case '%':                       // modulus
                 tempop = pop();
                 tempop2 = pop();
                 pushl(tempop2 % tempop);
                 if ((tempop2 % tempop) == 0)
                      printf("%d is dividable by %d\n", tempop2, tempop);
                 else
                      printf("%d is not dividable by %d\n", tempop2, tempop);
                 break;
            case '\n':
                 printf("\t%.30g\n", pop());
                 break;
            case 'c':                       // clear stack
                 stackc();
                 break;
            case 't':                       // celsius to fahrenheit
                 push(pop() * 1.8 + 32.0);
                 break;
            case 'h':                       // fahrenheit to celsius
                 push((pop() - 32.0) / 1.8);
                 break;
            case 'm':                       // metres to feet (english imperial)
                 push(pop() / 0.3048);
                 break;
            case 'f':                       // feet (english imperial) to metres
                 push(pop() * 0.3048);
                 break;            
            case 'y':
      printf("\nReverse Polish notation works by \"pushing\" read operands onto a stack. ");
      printf("Then it \"pops\" the values on the top of the stack when it encounters an operator; ");
      printf("the operator is applied to them, and the result is pushed back onto the stack. ");
      printf("Binary operators (+, -, *, /, %%, etc.) pop two values.\n\n");
      printf("To illustrate, let's write the expression (2 + 3) * (4 - 5) in reverse Polish.\n");
      printf("First, you'd want to add 3 to 2. To do that, you have to write: 2 3 +\n");
      printf("The program then reads the numbers, pushing them on the stack as it does so, ");
      printf("then pops them, in this case it adds 3 to 2.\n");
      printf("Next, you'd need to subtract 5 from 4, like this: 4 5 -\n");
      printf("Again, the program reads the numbers, pushing them on the stack, and then pops them.\n");
      printf("Lastly, you'd have to multiply the results, which you do by adding an * at the end.\n");
      printf("The two operands of * are now 5 and -1 from the previous calculations.\n");
      printf("The full expression is as follows: 2 3 + 4 5 - *\n\n");
      printf("There are several commands you can use in this program. They are as follows:\n\n");
      printf("NUMBER:\t\t\tNUMBER can be any number between -1e+308 and 1e+308\n\n");
      printf("Binary operators:\tBinary operators include +, -, *, /, %%.\n\t\t\t");
      printf("%% is a modulus operator.\n\t\t\tIn the expression x y %%, x is divided by y,\n\t\t\t");
      printf("and the remainder is returned.\n\t\t\tIf it is zero, x is dividable by y.\n\n");
      printf("Power:\t\t\tx y ^ is equivalent to x^y.\n\n");
      printf("Clear stack:\t\tThe command \'c\' clears the stack.\n\n");
      printf("Unit coversion:\t\tThe commands \'t\', \'h\', \'m\' and \'f\' convert\n\t\t\t");
      printf("Celsius to Fahrenheit, Fahrenheit to Celsius,\n\t\t\tMetres to Feet and Feet to Metres respectively.");
      printf("\n\t\t\tFeet are in English Imperial.\n\n");
            break;
            default:

                 printf("error: unknown command %s\n", s);
                 break;
            }
      }
      return 0;
}

#define MAXVAL 10000      // maximum depth of val stack

int sp = 0;               // next free stack position
double val[MAXVAL];       // value stack

/* push: push f onto value stack */
void push(double f)
{
     if (sp < MAXVAL)
          val[sp++] = f;
     else
          printf("error: stack full (10000+ operands/operators), can't push %g\n", f);
}

void pushl(long int f)
{
     if (sp < MAXVAL)
          val[sp++] = f;
     else
          printf("error: stack full (10000+ operands/operators), can't push %g\n", f);
}

/* pop: pop and return top value from stack */
double pop(void)
{
     if (sp > 0)
          return val[--sp];
     else {
          printf("error: stack empty\n");
          return 0.0;
     }
}      

/* stackc: clear stack */
void stackc(void)
{
     if (sp > 0) {
         while (sp >= 0) {
              printf("clearing %-20g\t#%05d...\n", val[sp], sp);
              val[sp--] = 0;
         }
          printf("stack cleared\n");
     } else
          printf("stack already empty\n");
}
     

#include <ctype.h>

int getch(void);
void ungetch(int);

/* getop: get next operator or numeric operand */
int getop(char s[])
{
    int i, c;
    
    while ((s[0] = c = getch()) == ' ' || c == '\t')
        ;
    s[1] = '\0';
    if (!isdigit(c) && c != '.')
        return c;      // not a number
    i = 0;
    if (isdigit(c))    // collect integer part
        while (isdigit(s[++i] = c = getch()))
            ;
    if (c == '.')      // collect fraction part
        while (isdigit(s[++i] = c = getch()))
            ;
    s[i] = '\0';
    if (c != EOF)
        ungetch(c);
    return NUMBER;
}

#define BUFSIZE 100

char buf[BUFSIZE];      // buffer for ungetch
int bufp = 0;           // next free position in buf

int getch(void)         /* get a (possibly pushed back) character */
{
    return (bufp > 0) ? buf[--bufp] : getchar();
}

void ungetch(int c)     /* push character back on input */
{
     if (bufp >= BUFSIZE)
          printf("ungetch: too many characters\n");
     else
          buf[bufp++] = c;
}
